@@ -1,11 +1,27 @@
+/*
+   Copyright The containerd Authors.
+
+   Licensed under the Apache License, Version 2.0 (the "License");
+   you may not use this file except in compliance with the License.
+   You may obtain a copy of the License at
+
+       http://www.apache.org/licenses/LICENSE-2.0
+
+   Unless required by applicable law or agreed to in writing, software
+   distributed under the License is distributed on an "AS IS" BASIS,
+   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+   See the License for the specific language governing permissions and
+   limitations under the License.
+*/
+
 package oci
 
 import (
 	"testing"
 
+	"github.com/gotestyourself/gotestyourself/assert"
 	"github.com/opencontainers/go-digest"
 	ocispec "github.com/opencontainers/image-spec/specs-go/v1"
-	"github.com/stretchr/testify/assert"
 )
 
 func TestNormalizeImageRef(t *testing.T) {
@@ -31,7 +47,7 @@ func TestNormalizeImageRef(t *testing.T) {
 		},
 	} {
 		normalized, err := normalizeImageRef(imageBaseName, test.input)
-		assert.NoError(t, err)
+		assert.NilError(t, err)
 		assert.Equal(t, test.expect, normalized)
 	}
 }
