@@ -1,3 +1,19 @@
+/*
+   Copyright The containerd Authors.
+
+   Licensed under the Apache License, Version 2.0 (the "License");
+   you may not use this file except in compliance with the License.
+   You may obtain a copy of the License at
+
+       http://www.apache.org/licenses/LICENSE-2.0
+
+   Unless required by applicable law or agreed to in writing, software
+   distributed under the License is distributed on an "AS IS" BASIS,
+   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+   See the License for the specific language governing permissions and
+   limitations under the License.
+*/
+
 package rootfs
 
 import (
@@ -10,11 +26,11 @@ import (
 	"golang.org/x/net/context"
 )
 
-// Diff creates a layer diff for the given snapshot identifier from the parent
-// of the snapshot. A content ref is provided to track the progress of the
-// content creation and the provided snapshotter and mount differ are used
+// CreateDiff creates a layer diff for the given snapshot identifier from the
+// parent of the snapshot. A content ref is provided to track the progress of
+// the content creation and the provided snapshotter and mount differ are used
 // for calculating the diff. The descriptor for the layer diff is returned.
-func Diff(ctx context.Context, snapshotID string, sn snapshots.Snapshotter, d diff.Differ, opts ...diff.Opt) (ocispec.Descriptor, error) {
+func CreateDiff(ctx context.Context, snapshotID string, sn snapshots.Snapshotter, d diff.Comparer, opts ...diff.Opt) (ocispec.Descriptor, error) {
 	info, err := sn.Stat(ctx, snapshotID)
 	if err != nil {
 		return ocispec.Descriptor{}, err
@@ -42,5 +58,5 @@ func Diff(ctx context.Context, snapshotID string, sn snapshots.Snapshotter, d di
 		defer sn.Remove(ctx, upperKey)
 	}
 
-	return d.DiffMounts(ctx, lower, upper, opts...)
+	return d.Compare(ctx, lower, upper, opts...)
 }
