@@ -1,5 +1,21 @@
 // +build !windows
 
+/*
+   Copyright The containerd Authors.
+
+   Licensed under the Apache License, Version 2.0 (the "License");
+   you may not use this file except in compliance with the License.
+   You may obtain a copy of the License at
+
+       http://www.apache.org/licenses/LICENSE-2.0
+
+   Unless required by applicable law or agreed to in writing, software
+   distributed under the License is distributed on an "AS IS" BASIS,
+   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+   See the License for the specific language governing permissions and
+   limitations under the License.
+*/
+
 package testutil
 
 import (
@@ -8,15 +24,14 @@ import (
 	"testing"
 
 	"github.com/containerd/containerd/mount"
-	"github.com/stretchr/testify/assert"
+	"github.com/gotestyourself/gotestyourself/assert"
 )
 
 // Unmount unmounts a given mountPoint and sets t.Error if it fails
-func Unmount(t *testing.T, mountPoint string) {
+func Unmount(t testing.TB, mountPoint string) {
 	t.Log("unmount", mountPoint)
-	if err := mount.UnmountAll(mountPoint, umountflags); err != nil {
-		t.Error("Could not umount", mountPoint, err)
-	}
+	err := mount.UnmountAll(mountPoint, umountflags)
+	assert.NilError(t, err)
 }
 
 // RequiresRoot skips tests that require root, unless the test.root flag has
@@ -24,7 +39,6 @@ func Unmount(t *testing.T, mountPoint string) {
 func RequiresRoot(t testing.TB) {
 	if !rootEnabled {
 		t.Skip("skipping test that requires root")
-		return
 	}
 	assert.Equal(t, 0, os.Getuid(), "This test must be run as root.")
 }
